<?php

/**
 * Define the internationalization functionality
 *
 * Loads and defines the internationalization files for this plugin
 * so that it is ready for translation.
 *
 * @link       https://themeforest.net/user/phpface
 * @since      1.0.0
 *
 * @package    Streamtube_Core
 * @subpackage Streamtube_Core/includes
 */

/**
 * Define the internationalization functionality.
 *
 * Loads and defines the internationalization files for this plugin
 * so that it is ready for translation.
 *
 * @since      1.0.0
 * @package    Streamtube_Core
 * @subpackage Streamtube_Core/includes
 * @author     phpface <nttoanbrvt@gmail.com>
 */
if( ! defined('ABSPATH' ) ){
    exit;
}

class Streamtube_Core_Taxonomy {

	/**
	 *
	 * Video Category taxonomy
	 * 
	 * @since 1.0.0
	 * 
	 */
	public function video_category(){
		$labels = array(
			"name" 									=> esc_html__( "Video Categories", "streamtube-core" ),
			"singular_name" 						=> esc_html__( "Video Category", "streamtube-core" ),
		);

		$args = array(
			"label" 								=> esc_html__( "Video Categories", "streamtube-core" ),
			"labels" 								=> $labels,
			"public" 								=> true,
			"publicly_queryable" 					=> true,
			"hierarchical" 							=> true,
			"show_ui" 								=> true,
			"show_in_menu" 							=> true,
			"show_in_nav_menus" 					=> true,
			"query_var" 							=> true,
			"rewrite" 								=> array(
				'slug' 			=> 'categories', 
				'with_front' 	=> true,  
				'hierarchical' 	=> true
			),
			"show_admin_column" 					=> true,
			"show_in_rest" 							=> false,
			"rest_base" 							=> "video_category",
			"rest_controller_class" 				=> "WP_REST_Terms_Controller",
			"show_in_quick_edit" 					=> true,
		);

		register_taxonomy( "categories", array( 'video' ), $args );		
	}

	/**
	 *
	 * Video Tag taxonomy
	 * 
	 * @since 1.0.0
	 * 
	 */
	public function video_tag(){

		$labels = array(
			"name" 									=> esc_html__( "Video Tags", "streamtube-core" ),
			"singular_name" 						=> esc_html__( "Video Tag", "streamtube-core" ),
		);

		$args = array(
			"label" 								=> esc_html__( "Video Tags", "streamtube-core" ),
			"labels" 								=> $labels,
			"public" 								=> true,
			"publicly_queryable" 					=> true,
			"hierarchical" 							=> false,
			"show_ui" 								=> true,
			"show_in_menu" 							=> true,
			"show_in_nav_menus" 					=> true,
			"query_var" 							=> true,
			"rewrite" 								=> array(
				'slug' 			=> 'video_tag', 
				'with_front' 	=> true
			),
			"show_admin_column" 					=> true,
			"show_in_rest" 							=> false,
			"rest_base" 							=> "video_tag",
			"rest_controller_class"					=> "WP_REST_Terms_Controller",
			"show_in_quick_edit" 					=> true,
		);

		register_taxonomy( "video_tag", array( 'video' ), $args );
	}	
}