<?php
/**
 * Define the video functionality
 *
 *
 * @link       https://themeforest.net/user/phpface
 * @since      1.0.0
 *
 * @package    Streamtube_Core
 * @subpackage Streamtube_Core/includes
 */

/**
 * Define the profile functionality
 *
 * @since      1.0.0
 * @package    Streamtube_Core
 * @subpackage Streamtube_Core/includes
 * @author     phpface <nttoanbrvt@gmail.com>
 */
if( ! defined('ABSPATH' ) ){
    exit;
}

class Streamtube_Core_Video {

	/**
	 * 
	 *
	 * Plugin instance.
	 * 
	 * @return [type] [description]
	 */
	private function plugin(){
		return streamtube_core()->get();
	}

	/**
	 *
	 * Filter video embed url
	 * 
	 * @param  string $embed_url
	 * @param  object $post
	 * @return string
	 *
	 * @since 1.0.0
	 * 
	 */
	public function filer_embed_url( $embed_url, $post ){
		return add_query_arg(
			array(
				'embed'	=>	'on'
			),
			wp_get_shortlink( $post->ID )
		);
	}

	/**
	 *
	 * Filter the embed html
	 * 
	 * @param  string $output
	 * @param  object $post
	 * @param  int $width
	 * @param  int $height
	 * @return string
	 *
	 * @since  1.0.0
	 * 
	 */
	public function filter_embed_html( $output, $post, $width, $height ){

		if( $post->post_type == 'video' ){

			$embed_url = get_post_embed_url( $post );

			if( did_action( 'streamtube/playlist/first_post/loaded' ) ){
				$embed_url = add_query_arg( array( 'logo' => '0' ), $embed_url );
			}

			$attrs = array(
				'width'				=>	$width,
				'height'			=>	$height,
				'src'				=>	$embed_url,
				'allow'				=>	'accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture; fullscreen'
			);

			/**
			 *
			 * Filter the embed iframe attributes
			 * 
			 * @param  object $post
			 * @param  int $width
			 * @param  int $height
			 * @return string
			 *
			 * @since  1.0.0
			 * 
			 */
			$attrs = apply_filters( 'streamtube_core_video_embed_iframe_attrs', $attrs, $post, $width, $height );

			if( ! $attrs ){
				return $output;
			}

			$_attrs = array();

			foreach ( $attrs as $attr => $value ) {
				$_attrs[] = sprintf( '%s="%s"', $attr, esc_attr( $value ) );
			}

			$output = sprintf(
				'<iframe %s></iframe>',
				join( ' ', $_attrs )
			);
		}

		return $output;
	}

	/**
	 *
	 * Load the share button
	 * 
	 * @since 1.0.0
	 */
	public function load_button_share(){
		load_template( streamtube_core_get_template( 'video/button-share.php' ) );
	}

	/**
	 *
	 * Load the share modal
	 * 
	 * @since 1.0.0
	 */
	public function load_modal_share(){
		load_template( streamtube_core_get_template( 'video/modal-share.php' ) );
	}

	/**
	 *
	 * Load single video post date 
	 * 
	 * @since 1.0.8
	 * 
	 */
	public function load_single_post_date(){
		get_template_part( 'template-parts/post-date-normal' );
	}

	/**
	 *
	 * Load single video post views 
	 * 
	 * @since 1.0.8
	 * 
	 */
	public function load_single_post_views(){

		if( ! $this->plugin()->analytics->is_connected() ){
			return;
		}
		get_template_part( 'template-parts/post-views', '', array( 'realtime' => false ) );
	}

	/**
	 *
	 * Load single video post comment count
	 * 
	 * @since 1.0.8
	 * 
	 */
	public function load_single_post_comment_count(){
		get_template_part( 'template-parts/post-comment', null, array( 'text' => true ) );
	}

	/**
	 *
	 * Load single video terms
	 * 
	 * @since 1.0.8
	 * 
	 */
	public function load_single_post_terms(){
		get_template_part( 'template-parts/post-term', null, array(
            'taxonomy'  =>  'categories'
        ) );
	}

	/**
	 *
	 * Load the single post analytics button
	 *
	 * This button redirect to dashboard page.
	 * 
	 * @return output HTML
	 *
	 * @since 1.0.8
	 * 
	 */
	public function load_single_post_analytics_button(){

		global $post;

		if( ! get_option( 'sitekit_reports', 'on' ) || ! current_user_can( 'edit_post', $post->ID ) || ! $this->plugin()->analytics->is_connected() ){
			return;
		}

		$path = "dashboard/{$post->post_type}/{$post->ID}/analytics/";

		$url = trailingslashit( get_author_posts_url( $post->post_author ) ) . $path;
		?>
		<div class="button-group">
			<button onclick="location.href = '<?php echo esc_url( $url )?>';" class="btn px-3 shadow-none d-flex align-items-center">
				<span class="btn__icon icon-chart-area"></span>
				<span class="btn__text small text-secondary">
					<?php esc_html_e( 'Analytics', 'streamtube-core' )?>
				</span>
			</button>
		</div>
		<?php
	}
}