(function($) {
    "use strict";

    $.ajaxSetup({
        headers : {
            'X-WP-Nonce' : streamtube.nonce
        }
    });

    $( document.body ).on( 'load_analytics_overview_done', loadAnalyticsOverviewTabs );

    $( document.body ).on( 'analytics_overview_tabs_loaded', loadAnalyticsOverviewCharts );

    $( document.body ).on( 'load_analytics_overview_video_done', loadAnalyticsOverviewVideoTabs );

        $( document.body ).on( 'analytics_overview_video_tabs_loaded', loadAnalyticsVideoCharts );
  
    $( document ).on( 'shown.bs.tab', '.analytics-tab[data-bs-toggle="tab"]', function( event ){

        var tab = event.target.getAttribute( 'data-bs-target' ).replace( '#', '' );
        var params = $.parseJSON( $( '#analytics-overview' ).attr( 'data-params' ) );

        return tabSwitcher( tab, params );
    } );    

    function showError( message, elementObject ){
        var output = '<div class="position-absolute top-50 start-50 translate-middle">';
            output += '<div class="msg text-muted">'+ message +'</div>';
        output += '</div>'
        return elementObject.html( output);
    }

    /**
     *
     * Convert given number to Session Duration format
     */
    function convertNumberToSessionDuration( value ){
        const sec = parseInt(value, 10);
        let hours   = Math.floor(sec / 3600);
        let minutes = Math.floor((sec - (hours * 3600)) / 60);
        let seconds = sec - (hours * 3600) - (minutes * 60);
    
        if( hours == 0 ){
            if( minutes > 0 ){
                return minutes + analytics.minute + ' ' + seconds + analytics.second;
            }
            else{
                return seconds + analytics.second;
            }
        }

        return hours + analytics.hour + ' ' + minutes + analytics.minute + ' ' + seconds + analytics.second;
    }

    /**
     *
     * Get  google search console with given keyword and date range
     *
     * @return string
     * 
     * @since 1.0.8
     */
    function getGoogleSearchConsoleUrl( keyword, start_date = '', end_date = '' ){
        var url = 'https://search.google.com/search-console/performance/search-analytics';
        url += '?start_date=' + start_date.replace(/([-])+/g, '' );
        url += '&end_date=' + end_date.replace(/([-])+/g, '' );
        url += '&query=!' + keyword;
        url += '&resource_id=' + analytics.home_url;
        return url;
    }

    /**
     *
     * format date
     * 
     * @param string date yyymmdd
     */
    function formatDate( date ){
        return date.replace( /(\d{4})(\d{2})(\d{2})/g, "$1-$2-$3" ).split("-");
    }

    function getSessionStorage( dataPoint = '', params = [] ){

        if( window.sessionStorage == undefined || ! analytics.session_storage ){
            return false;
        }

        params.dataPoint = dataPoint;
        params.userid = analytics.user_id;

        var sessionId = btoa(JSON.stringify( params ));

        var session = sessionStorage.getItem( sessionId );

        if( session ){
            return $.parseJSON( session );
        }

        return false;   
    }

    function setSessionStorage( data, dataPoint = '', params = [] ){

        if( window.sessionStorage == undefined || ! analytics.session_storage ){
            return false;
        }

        params.dataPoint = dataPoint;
        params.userid = analytics.user_id;

        var sessionId = btoa(JSON.stringify( params ));

        return sessionStorage.setItem( sessionId, JSON.stringify(data) );
    }

    /**
     *
     * Draw LineChart from given Rest Response
     * 
     * @param  array response
     * @param  string elementId
     * @return Draw the line chart
     * 
     */
    function drawLineChart( response, elementId ){

        var name    = response.data.response[0].columnHeader.metricHeader.metricHeaderEntries[0].name; 
        var rows    = response.data.response[0].data.rows;
        var requestId = response.data.requestId;

        google.charts.load( 'current', {
                'packages':['corechart'], 
                'language' : streamtube.language
            }
        );

          google.charts.setOnLoadCallback(_drawLineChart);

          function _drawLineChart() {

            var dataTable = new google.visualization.DataTable();

            dataTable.addColumn( 'date', 'Date' );
            dataTable.addColumn( 'number', name );
            dataTable.addColumn( 'number', analytics.previous_period );
            dataTable.addColumn( {'type': 'string', 'role': 'tooltip', 'p': {'html': true}} );

            var length = rows.length;

            for ( var i = 0; i < length/2; i++ ) {

                var date = new Date( formatDate( rows[i].dimensions[0] ) );
                var value = rows[i].metrics[1] != undefined ? rows[i].metrics[1].values[0] : 0;

                dataTable.addRow( [
                        date,
                        0,
                        Math.floor( value, 2 ),
                        ''
                    ]
                );
            }

            for ( var i = length/2; i < length; i++ ) {

                dataTable.setCell( i - length/2, 0, new Date( formatDate( rows[i].dimensions[0] ) ) );

                dataTable.setCell( i - length/2, 1, Math.floor( rows[i].metrics[0].values[0], 2 ) );

                var last        = Math.floor(rows[i].metrics[0].values[0], 2);

                var previous    = rows[i - length/2].metrics[1] != undefined ? Math.floor(rows[i - length/2].metrics[1].values[0],2) : 0;
                var percent     = 100;

                if( last == 0 && previous == 0 ){
                    percent = 0;
                }else{
                    if( previous == 0 ){
                        percent = 100;
                    }
                    else{
                        percent = last*100/previous - 100;    
                    }
                }

                if( $.isNumeric( percent ) ){
                    percent = Math.floor( percent, 2 );
                }
                else{
                    percent = 0;
                }
                
                var percentText = '';

                var downUp = percent < 0 ? 'down' : 'up';

                percentText += '<div class="percent percent-icon-'+downUp+' mt-3">';

                    percentText += '<span class="icon-'+downUp+'"></span>';

                    percentText += '<span>'+ percent.toString() +'%</span>';

                percent += '</div>';

                if( requestId == 'bouncerate' ){
                    previous = previous + '%';
                }

                if( requestId == 'avgsessionduration' ){
                    previous = convertNumberToSessionDuration( previous );
                }

                dataTable.setCell( i - length/2, 3, previous + percentText );
            }

            if( requestId == 'bouncerate' ){
                var formatter = new google.visualization.NumberFormat( {
                    fractionDigits: 0,
                    suffix:'%'
                } );

                formatter.format( dataTable, 1 );
            }

            if( requestId == 'avgsessionduration' ){
                var formatter = new google.visualization.NumberFormat( );

                formatter.format( dataTable, 1 );
            }

            var chart = new google.visualization.LineChart(document.getElementById( elementId ));

            chart.draw( dataTable , $.parseJSON( $( '#googlesitekit-reports' ).attr( 'data-linechart-options' ) ) );
        }
    }

    /**
     *
     * Draw pageViews report
     * 
     * @param  array params
     * 
     */
    function drawPageViewsReports( params, elementId ){

        var dataPoint = 'pageviews';

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return drawLineChart( session, elementId ); 
        }

        // Get pageViews reports
        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function(response){
            
            if( response.success == false ){
                return showError( response.data[0].message, $( '#' + elementId ) );
            }

            setSessionStorage( response, dataPoint, params );

            return drawLineChart( response, elementId );
        } );
    }

    /**
     *
     * Draw uniquePageViews report
     * 
     * @param  array params
     * 
     */
    function drawUniquePageViewsReports( params, elementId ){
        // Get uniquePageViews reports
        
        var dataPoint = 'uniquepageviews';

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return drawLineChart( session, elementId ); 
        }        

        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function(response){
            if( response.success == false ){
                return showError( response.data[0].message, $( '#' + elementId ) );
            }

            setSessionStorage( response, dataPoint, params );

            return drawLineChart( response, elementId );
        } );
    }

    /**
     *
     * Draw Users reports chart
     * 
     * @param  array params
     * 
     */
    function drawUsersReports( params, elementId ){
        // Get users reports
        
        var dataPoint = 'users';

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return drawLineChart( session, elementId ); 
        }

        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function(response){
            if( response.success == false ){
                return showError( response.data[0].message, $( '#' + elementId ) );
            }

            setSessionStorage( response, dataPoint, params );

            return drawLineChart( response, elementId );
        } );        
    }

    /**
     *
     * Draw sessions reports chart
     * 
     * @param  array params
     * 
     */
    function drawSessionsReports( params, elementId ){

        var dataPoint = 'sessions';

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return drawLineChart( session, elementId ); 
        }

        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function(response){
            if( response.success == false ){
                return showError( response.data[0].message, $( '#' + elementId ) );
            }            

            setSessionStorage( response, dataPoint, params );

            return drawLineChart( response, elementId );      
        } );
    }

    /**
     *
     * Draw sessions reports chart
     * 
     * @param  array params
     * 
     */
    function drawBounceRateReports( params, elementId ){
        var dataPoint = 'bouncerate';

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return drawLineChart( session, elementId ); 
        }

        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function(response){
            if( response.success == false ){
                return showError( response.data[0].message, $( '#' + elementId ) );
            }            

            setSessionStorage( response, dataPoint, params );

            return drawLineChart( response, elementId );      
        } );
    }

    /**
     *
     * Draw Session Duration reports chart
     * 
     * @param  array params
     * 
     */
    function drawAvgSessionDurationReports( params, elementId ){

        var dataPoint = 'avgsessionduration';

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return drawLineChart( session, elementId ); 
        }

        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function(response){
            if( response.success == false ){
                return showError( response.data[0].message, $( '#' + elementId ) );
            }            

            setSessionStorage( response, dataPoint, params );

            return drawLineChart( response, elementId );      
        } );
    }

    function drawTotalVideoViewsReports( params, elementId ){
        
        var dataPoint = 'videoviews';

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return drawLineChart( session, elementId ); 
        }        

        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function(response){
            if( response.success == false ){
                return showError( response.data[0].message, $( '#' + elementId ) );
            }
            setSessionStorage( response, dataPoint, params );

            return drawLineChart( response, elementId );
        } );        
    }

    function drawUniqueVideoViewsReports( params, elementId ){
        var dataPoint = 'uniquevideoviews';

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return drawLineChart( session, elementId ); 
        }    

        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function(response){
            if( response.success == false ){
                return showError( response.data[0].message, $( '#' + elementId ) );
            }
            setSessionStorage( response, dataPoint, params );

            return drawLineChart( response, elementId );
        } );
    }

    /**
     *
     * Get overview handler
     * 
     */
    function loadAnalyticsOverviewTabs( event, response, params, wrapper, textStatus, jqXHR ){

        if( response.success == false ){
            return showError( response.data[0].message, wrapper );
        }

        var tabs = '';
        var tabsContent = '';        

        var metrics = $.parseJSON( $( '#googlesitekit-reports' ).attr( 'data-overview-metrics' ) );

        var data        = response.data.response[0].data.totals;
        var columnHeader = response.data.response[0].columnHeader.metricHeader.metricHeaderEntries;

        //console.log(columnHeader);

        for ( var i = 0; i < data[0].values.length; i++ ) {

            var active = '';

            if( i == 0 ){
                active = 'active show';
            }

            var number1 = parseInt( data[0].values[i] );

            var number2 = false;

            if( data[1] !== undefined ){
                number2 = parseInt( data[1].values[i] );    
            }

            var percent = 100 - ( number2*100/number1 );

            if( ! $.isNumeric( percent ) ){
                percent = 0;
            }

            var icon = number1 > number2 ? 'icon-up' : 'icon-down';

            var tabId = Object.keys(metrics)[i];

            if( tabId == 'bouncerate' ){
                number1 = number1 + '%';
            }

            if( tabId == 'avgsessionduration' ){
                number1 = convertNumberToSessionDuration( number1 );
            }            

            // Tab selector
            tabs += '<li class="nav-item flex-fill" role="presentation">';
                tabs += '<div class="nav-link '+active+' analytics-tab" id="analytics-tab-'+tabId+'" data-bs-toggle="tab" data-bs-target="#analytics-content-'+tabId+'">';
                    tabs += '<h3 class="analytics-tab__label text-secondary">'+ columnHeader[i]['name'] +'</h3>';
                    tabs += '<div class="analytics-tab__number total-number text-body"><h3>'+ number1.toLocaleString() +'</h3></div>';
                    tabs += '<div class="analytics-tab__percent percent percent-'+icon+'">';
                        tabs += '<span class="analytics-tab__icon '+icon+'"></span>';
                        tabs += '<span class="analytics-tab__percent-number percent-number">'+ Math.floor( percent,2 ) +'%</span>';
                    tabs += '</div>';
                tabs += '</div>';
            tabs += '</li>'; 

            // Tab content
            tabsContent += '<div class="tab-pane fade '+ active +'" id="analytics-content-'+tabId+'" role="tabpanel">';
                tabsContent += '<div class="position-relative" id="analytics-chart-'+i+'" style="width: 100%; height: 400px;">';
                    tabsContent += '<div class="position-absolute top-50 start-50 translate-middle">';
                        tabsContent += '<div class="spinner-border text-secondary" role="status">';
                            tabsContent += '<span class="visually-hidden">Loading...</span>';
                        tabsContent += '</div>';
                    tabsContent += '</div>';
                tabsContent += '</div>';
            tabsContent += '</div>';

            $( document.body ).trigger( 'analytics_overview_tab_'+tabId+'_loaded', [ response, params, wrapper, textStatus, jqXHR ] );
        }
        // end for;

        tabs = '<ul class="nav nav-tabs analytics-tabs" id="analytics-overview-report-tabs" role="tablist">'+ tabs +'</ul>';

        tabsContent = '<div class="tab-content mt-5" id="analytics-tabs-content">'+ tabsContent +'</div>';

        wrapper.html( tabs + tabsContent );

        $( document.body ).trigger( 'analytics_overview_tabs_loaded', [ response, params, wrapper, textStatus, jqXHR ] );
    }

    /**
     * Load overview charts
     */
    function loadAnalyticsOverviewCharts( event, response, params, wrapper, textStatus, jqXHR ){
        // Load first chart.
        var tab = wrapper.find( 'ul#analytics-overview-report-tabs li:first-child .nav-link' ).attr( 'data-bs-target' );

        return tabSwitcher( tab.replace( '#', '' ), params );
    }

    /**
     *
     * Fires after overview tabs loaded
     * 
     */
    function loadAnalyticsVideoOverview( wrapper ){

        if( wrapper.length == 0 ){
            return;
        }     

        var action      =   'load_analytics_overview_video';

        var params = $.parseJSON( wrapper.attr( 'data-params' ) );

        var dataPoint = 'videooverviews';

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return $( document.body ).trigger( action + '_done', [ session, params, wrapper ] );
        }

        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function(response){

            if( response.success == false ){
                return $( '#analytics-overview-videos .spinner-wrap' ).html( '<div class="msg text-muted">' + response.data[0].message + '</div>' );
            }

            setSessionStorage( response, dataPoint, params );

            $( document.body ).trigger( action + '_done', [ response, params, wrapper ] );
        } );
    }

    function loadAnalyticsOverviewVideoTabs( event, response, params, wrapper ){
        var tabs = '';
        var tabsContent = '';

        var metrics = $.parseJSON( $( '#googlesitekit-reports' ).attr( 'data-overview-playevent-metrics' ) );

        var data            = response.data.response[0].data.totals;
        var columnHeader    = response.data.response[0].columnHeader.metricHeader.metricHeaderEntries;

        for ( var i = 0; i < data[0].values.length; i++ ) {

            var active = '';

            if( i == 0 ){
                active = 'active show';
            }

            var number1 = parseInt( data[0].values[i] );

            var number2 = false;

            if( data[1] !== undefined ){
                number2 = parseInt( data[1].values[i] );    
            }

            var percent = 100 - ( number2*100/number1 );

            if( ! $.isNumeric( percent ) ){
                percent = 0;
            }

            var icon = number1 > number2 ? 'icon-up' : 'icon-down';

            var tabId = Object.keys(metrics)[i];           

            // Tab selector
            tabs += '<li class="nav-item flex-fill" role="presentation">';
                tabs += '<div class="nav-link '+active+' analytics-tab" id="analytics-video-tab-'+tabId+'" data-bs-toggle="tab" data-bs-target="#analytics-video-content-'+tabId+'">';
                    tabs += '<h3 class="analytics-tab__label text-secondary">'+ columnHeader[i]['name'] +'</h3>';
                    tabs += '<div class="analytics-tab__number total-number text-body"><h3>'+ number1.toLocaleString() +'</h3></div>';
                    tabs += '<div class="analytics-tab__percent percent percent-'+icon+'">';
                        tabs += '<span class="analytics-tab__icon '+icon+'"></span>';
                        tabs += '<span class="analytics-tab__percent-number percent-number">'+ Math.floor( percent,2 ) +'%</span>';
                    tabs += '</div>';
                tabs += '</div>';
            tabs += '</li>'; 

            // Tab content
            tabsContent += '<div class="tab-pane fade '+ active +'" id="analytics-video-content-'+tabId+'" role="tabpanel">';
                tabsContent += '<div class="position-relative" id="analytics-chart-'+i+'" style="width: 100%; height: 400px;">';
                    tabsContent += '<div class="position-absolute top-50 start-50 translate-middle">';
                        tabsContent += '<div class="spinner-border text-secondary" role="status">';
                            tabsContent += '<span class="visually-hidden">Loading...</span>';
                        tabsContent += '</div>';
                    tabsContent += '</div>';
                tabsContent += '</div>';
            tabsContent += '</div>';
        }

        tabs = '<ul class="nav nav-tabs analytics-tabs" id="google-analytics-video-contents" role="tablist">'+ tabs +'</ul>';

        tabsContent = '<div class="tab-content mt-5" id="analytics-video-tabs-contents-wrap">'+ tabsContent +'</div>';

        var output = '<div class="analytics-section__content analytics-section-videoviews position-relative border-top pt-4">'
            output += tabs + tabsContent;
        output += '</div>';

        wrapper.html( output );

        $( document.body ).trigger( 'analytics_overview_video_tabs_loaded', [ response, params, wrapper ] );
    }

    function loadAnalyticsVideoCharts( event, response, params, wrapper ){
        // Load pageViews first.
        drawTotalVideoViewsReports( params, 'analytics-video-content-videoviews' );        
    }

    /**
     *
     * Loads top pageViews
     * 
     */
    function loadAnalyticsTopPageViews( wrapper ){

        if( wrapper.length == 0 ){
            return;
        }        

        var params = $.parseJSON( wrapper.attr( 'data-params' ) );

        var dataPoint = 'toppageviews';

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return _loadAnalyticsTopPageViews( session );
        }        

        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function( response ){

            if( response.success == false ){
                return showError( response.data[0].message, wrapper.find( '#analytics-top-pageviews' ) );
            }

            setSessionStorage( response, dataPoint, params );

            return _loadAnalyticsTopPageViews( response );
        } );

        function _loadAnalyticsTopPageViews( response ){
            var rows = response.data.response[0].data.rows;

            if( rows == undefined ){
                return showError( analytics.data_not_available, wrapper.find( '#analytics-top-pageviews' ) );
            }

            var columnHeader = response.data.response[0].columnHeader.metricHeader.metricHeaderEntries;

            var table = '';

            table += '<table class="table">';

                table += '<thead>';

                    table += '<tr>';

                        table += '<th scope="col">'+ analytics.title +'</th>';

                        for ( var i = 0; i < columnHeader.length; i++ ) {
                            table += '<th scope="col">'+ columnHeader[i].name +'</th>';
                        }
       
                    table += '</tr>';

                table += '<thead><!--thead-->';

                table += '<tbody>';

                    for ( var i = 0; i < rows.length; i++ ) {
                        var path            = rows[i].dimensions[0];
                        var permalink       = analytics.home_url + path;
                        var title           = rows[i].dimensions[1];
                        var pageViews       = parseInt( rows[i].metrics[0].values[0] );
                        var uniquePageViews = parseInt( rows[i].metrics[0].values[1] );
                        var boundRate       = rows[i].metrics[0].values[2];

                        table += '<tr>';                 

                            table += '<th scope="row">';
                                table += '<div class="item-permalink"><a href="'+ permalink +'" target="_blank">'+ title +'</a></div>';
                                table += '<div class="item-path"><a class="small text-secondary" href="'+ permalink +'" target="_blank">'+ path +'</a></div>';
                            table += '</th>';

                            table += '<td data-title="'+ columnHeader[0].name +'">'+ pageViews.toLocaleString() +'</td>';
                            table += '<td data-title="'+ columnHeader[1].name +'">'+ uniquePageViews.toLocaleString() +'</td>';
                            table += '<td data-title="'+ columnHeader[2].name +'">'+ Math.floor( boundRate, 2 ) +'%</td>';

                        table += '</tr>';
                    }

                table += '</tbody>';

            table += '</table>';

            wrapper.find( '#analytics-top-pageviews' ).html( table );
        }
    }

    function loadAnalyticsChannels( wrapper ){

        if( wrapper.length == 0 ){
            return;
        }        

        var dataPoint = 'topchannels';

        var params = $.parseJSON( wrapper.attr( 'data-params' ) );

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return _loadAnalyticsChannels( session );
        }        

        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function( response ){
            if( response.success == false ){
                return showError( response.data[0].message, wrapper.find( '#analytics-channels' ) );
            }

            setSessionStorage( response, dataPoint, params );

            return _loadAnalyticsChannels( response );
        });

        function _loadAnalyticsChannels( response ){
            var rowTable        = [];
            var columnHeader    = response.data.response[0].columnHeader.metricHeader.metricHeaderEntries; 
            var rows            = response.data.response[0].data.rows;
            var totals          = response.data.response[0].data.totals[0].values;
            var totalsNumber    = 0;

            if( rows == undefined ){
                return showError( analytics.data_not_available, wrapper.find( '#analytics-channels' ) );
            }            

            for (var i = 0; i < totals.length; i++) {
                totalsNumber += parseInt( totals[i] );
            }

            var table           = '';

            table += '<table class="table">';

                table += '<thead>';

                    table += '<tr>';

                        table += '<th scope="col">'+ analytics.channel +'</th>';

                        for ( var i = 0; i < columnHeader.length; i++ ) {
                            table += '<th scope="col">'+ columnHeader[i].name +'</th>';
                        }

                        table += '<th scope="col">'+ analytics.percentage +'</th>';
       
                    table += '</tr>';

                table += '<thead><!--thead-->';

                table += '<tbody>';

                for ( var i = 0; i < rows.length; i++ ) {  

                    var channel    = rows[i].dimensions[0];
                    var users      = parseInt( rows[i].metrics[0].values[0] );
                    var newUsers   = parseInt( rows[i].metrics[0].values[1] );
                    var sessions   = parseInt( rows[i].metrics[0].values[2] );
                    var total      = users+newUsers+sessions;
                    var percentage = total*100/totalsNumber;

                    rowTable.push( [ channel, percentage ] );

                    table += '<tr>';
                        table += '<th scope="row">'+ channel +'</th>';
                        table += '<td data-title="'+columnHeader[0].name+'">'+ users.toLocaleString() +'</td>';
                        table += '<td data-title="'+columnHeader[1].name+'">'+ newUsers.toLocaleString() +'</td>';
                        table += '<td data-title="'+columnHeader[2].name+'">'+ sessions.toLocaleString() +'</td>';
                        table += '<td data-title="'+analytics.percentage+'">'+ percentage.toFixed(2) +'%</td>';
                    table += '</tr>';
                }

                table += '</tbody>';

            table += '</table>';

            $( '#analytics-channel-table' ).html( table );

            google.charts.load('current', {'packages':['corechart']});
            google.charts.setOnLoadCallback(_drawChart);

            function _drawChart() {

                var dataTable = new google.visualization.DataTable();

                dataTable.addColumn( 'string', analytics.channel );
                dataTable.addColumn( 'number', analytics.percentage );

                for ( var i = 0; i < rowTable.length; i ++ ) {
                    dataTable.addRow( [ rowTable[i][0], rowTable[i][1] ] );
                }

                var chart = new google.visualization.PieChart(document.getElementById('analytics-channel-charts'));

                chart.draw( dataTable, $.parseJSON( wrapper.find( '#analytics-channel-charts' ).attr( 'data-chart-options' ) ) );
            }

            wrapper.find( '#analytics-channels .spinner-wrap' ).hide();
        }
    }

    function loadAnalyticsTopCountries( wrapper ){

        if( wrapper.length == 0 ){
            return;
        }        

        var dataPoint = 'topcountries';

        var params = $.parseJSON( wrapper.attr( 'data-params' ) );

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return _loadAnalyticsTopCountries( session );
        }

        $.get( analytics.rest_url + '/analytics/reports/'+dataPoint+'?' + $.param( params ),function( response ){

            if( response.success == false ){
                return showError( response.data[0].message, wrapper.find( '#analytics-countries' ) );
            }

            setSessionStorage( response, dataPoint, params );

            return _loadAnalyticsTopCountries( response );
        });

        function _loadAnalyticsTopCountries( response ){
            var rows      = response.data.response[0].data.rows;

            if( rows == undefined ){
                return showError( analytics.data_not_available, wrapper.find( '#analytics-countries' ) );
            }            

            google.charts.load('current', {
                'packages':['geochart'],
                'mapsApiKey': analytics.mapapikey
            });

            google.charts.setOnLoadCallback( drawGeoChart );

            function drawGeoChart(){
                var dataTable = new google.visualization.DataTable();

                dataTable.addColumn( 'string', analytics.country );
                dataTable.addColumn( 'number', analytics.users );

                for ( var i = 0; i < rows.length; i++ ) {
                    var country = rows[i].dimensions[0];
                    var users   = parseInt( rows[i].metrics[0].values[0] );
                    dataTable.addRow( [ country, users ] );
                }

                var options = $.parseJSON( $( '#analytics-countries-geo-chart' ).attr( 'data-chart-options' ) );

                var chart = new google.visualization.GeoChart(document.getElementById('analytics-countries-geo-chart'));

                chart.draw( dataTable, options );                
            }

            google.charts.load('current', {'packages':['corechart']});
            google.charts.setOnLoadCallback( drawPiechart );

            function drawPiechart(){
                var dataTable = new google.visualization.DataTable();

                dataTable.addColumn( 'string', analytics.country );
                dataTable.addColumn( 'number', analytics.users );

                for ( var i = 0; i < rows.length; i++ ) {
                    var country = rows[i].dimensions[0];
                    var users   = parseInt( rows[i].metrics[0].values[0] );
                    dataTable.addRow( [ country, users ] );
                }                
                var options = $.parseJSON( wrapper.find( '#analytics-countries-pie-chart' ).attr( 'data-chart-options' ) );

                var chart = new google.visualization.PieChart(document.getElementById('analytics-countries-pie-chart'));

                chart.draw( dataTable, options );                
            }
        }
    }

    function loadSearchQueriesKeywords( wrapper ){

        if( wrapper.length == 0 ){
            return;
        }

        var dataPoint = 'searchanalytics';

        var params = $.parseJSON( wrapper.attr( 'data-params' ) );

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return _loadSearchQueriesKeywords( session );
        }        

        $.get( analytics.rest_url + '/search-console/'+dataPoint+'?' + $.param( params ),function( response ){
            if( response.success == false ){
                return showError( response.data[0].message, wrapper.find( '#analytics-search-queries' ) );
            }

            setSessionStorage( response, dataPoint, params );

            return _loadSearchQueriesKeywords( response );
        });

        function _loadSearchQueriesKeywords( response ){
            var data = response.data.response;
            var params = response.data.params;
            var table = '';

            table += '<table class="table">';
                table += '<thead><tr>';
                    table += '<th scope="col">' + analytics.keyword + '</th>';
                    table += '<th scope="col">' + analytics.clicks + '</th>';
                    table += '<th scope="col">' + analytics.impressions + '</th>';
                    table += '<th scope="col">' + analytics.ctr + '</th>';
                    table += '<th scope="col">' + analytics.position + '</th>';
                table += '</tr></thead>';

                table += '<tbody>';
                    if( data.length > 0 ){
                        for ( var i = 0; i < data.length ; i++ ) {
                            var url = getGoogleSearchConsoleUrl( data[i].keys, params.startDate, params.endDate );
                            table += '<tr>';
                                table += '<th scope="row">';
                                    table += '<a class="fw-bold" href="'+ url +'" target="_blank">'+ data[i].keys +'</a>';  
                                table += '</td>';
                                table += '<td data-title="'+ analytics.clicks +'">'+ data[i].clicks.toLocaleString() +'</td>';
                                table += '<td data-title="'+ analytics.impressions +'">'+ data[i].impressions.toLocaleString() +'</td>';
                                table += '<td data-title="'+ analytics.ctr +'">'+ data[i].ctr +'</td>';
                                table += '<td data-title="'+ analytics.position +'">'+ Math.floor( data[i].position, 2 ) +'</td>';
                            table += '</tr>';
                        }
                    }
                    else{
                        table += '<tr>';
                            table += '<td colspan="5">';
                                table += '<p class="text-muted">'+ analytics.no_keywords_found +'</p>';
                            table += '</td>';
                        table += '</tr>';                        
                    }
                table += '</tbody>';

            table += '</table>';

            wrapper.find( '#analytics-search-queries' ).html( table );
        }
    }

    /**
     *
     * Analytics Overview Loader
     * 
     */
    function loadAnalyticsOverview( wrapper ){

        if( wrapper.length == 0 ){
            return;
        }

        var params      =   $.parseJSON( wrapper.attr( 'data-params' ) );

        var action      =   'load_analytics_overview';

        var dataPoint = 'overview';

        var session = getSessionStorage( dataPoint, params );

        if( session ){
            return $( document.body ).trigger( action + '_done', [ session, params, wrapper ] );
        }

        var jqxhr = $.ajax({
            url             : analytics.rest_url + '/analytics/reports/overview?' + $.param( params ),
            type            : 'GET',       
            beforeSend      : function( jqXHR ) {
                jqXHR.setRequestHeader( 'X-WP-Nonce', streamtube.nonce );
            }
        })
        .fail( function( jqXHR, textStatus, errorThrown ){
            $( document.body ).trigger( action + '_failed', [ jqXHR, textStatus, errorThrown ] );

            if( jqXHR.responseJSON.message !== undefined ){
                $.showToast( jqXHR.responseJSON.message , 'danger' );
            }
            else if( jqXHR.responseJSON.data[0].message ){
                $.showToast( jqXHR.responseJSON.data[0].message , 'danger' );
            }
            else{
                $.showToast( errorThrown, 'danger' );
            }
        })
        .done( function( response, textStatus, jqXHR ){

            if( response.success == true ){

                setSessionStorage( response, dataPoint, params );

                $( document.body ).trigger( action + '_done', [ response, params, wrapper, textStatus, jqXHR ] );
            }else{
                wrapper.find( '.spinner-wrap' ).html( response.data[0].message );
            }
            
        })
        .always( function( jqXHR, textStatus ){
            $( document.body ).trigger( action + '_completed', [ jqXHR, textStatus ] );
        });
    };

    /**
     *
     * Tab switcher
     * @since 1.0.8
     */
    function tabSwitcher( tab, params ){
        switch(tab){
            case 'analytics-content-pageviews':
                return drawPageViewsReports( params , tab );
            break;            

            case 'analytics-content-uniquepageviews':
                return drawUniquePageViewsReports( params , tab );
            break;            

            case 'analytics-content-users':
                return drawUsersReports( params , tab );
            break;

            case 'analytics-content-sessions':
                return drawSessionsReports( params , tab );
            break;

            case 'analytics-content-bouncerate':
                return drawBounceRateReports( params , tab );
            break;

            case 'analytics-content-avgsessionduration':
                return drawAvgSessionDurationReports( params , tab );
            break;

            case 'analytics-video-content-videoviews':
                return drawTotalVideoViewsReports( params , tab );
            break;

            case 'analytics-video-content-uniquevideoviews':
                return drawUniqueVideoViewsReports( params , tab );
            break;
        }
    }

    /**
     * chartInit
     */
    function siteKitAnalyticsInit(){
        loadAnalyticsOverview( $( '#analytics-overview' ) );
        loadAnalyticsVideoOverview( $( '#analytics-overview-videos' ) );

        loadAnalyticsTopPageViews( $( '#section-report-toppageviews' ) );
        loadAnalyticsChannels( $( '#section-report-channels' ) );
        loadAnalyticsTopCountries( $( '#section-report-countries' ) );
        loadSearchQueriesKeywords( $( '#section-report-search-queries' ) );        
    }

    // Run the loader
    siteKitAnalyticsInit();

    $( window ).resize(function() {
        loadAnalyticsOverview( $( '#analytics-overview' ) );
        loadAnalyticsVideoOverview( $( '#analytics-overview-videos' ) );

        loadAnalyticsTopPageViews( $( '#section-report-toppageviews' ) );
        loadAnalyticsChannels( $( '#section-report-channels' ) );
        loadAnalyticsTopCountries( $( '#section-report-countries' ) );
        loadSearchQueriesKeywords( $( '#section-report-search-queries' ) );
    });
})(jQuery);