<?php

/**
 * DB
 *
 * @link       https://themeforest.net/user/phpface
 * @since      1.0.0
 *
 * @package    WP_Post_Like
 * @subpackage WP_Post_Like/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    WP_Post_Like
 * @subpackage WP_Post_Like/includes
 * @author     phpface <nttoanbrvt@gmail.com>
 */
if( ! defined( 'ABSPATH' ) ){
	exit;
}

class WP_Post_Like_Query {

	/**
	 *
	 * Hold the table name
	 * 
	 * @var string
	 *
	 * @since  1.0.0
	 * 
	 */
	protected $table			=	'post_like';

	/**
	 *
	 * Hold the data version
	 * 
	 * @var string
	 *
	 * @since  1.0.0
	 * 
	 */
	protected $_db_version		=	'1.0.0';

	/**
	 *
	 * Get the table name include WP prefix.
	 * 
	 * @return string
	 *
	 * @since  1.0.0
	 * 
	 */
	public function get_table( $name = '' ){
		global $wpdb;

		if( empty( $name ) ){
			return $wpdb->prefix . $this->table;
		}

		return $wpdb->prefix . $this->table . '_' . $name;
	}

	/**
	 *
	 * Get charset collate
	 * 
	 * @return string
	 *
	 * @since  1.0.0
	 * 
	 */
	protected function get_charset_collate(){
		global $wpdb;
		return $wpdb->get_charset_collate();
	}

	/**
	 *
	 * Create reaction table
	 *
	 * @return  maybe_create_table();
	 * 
	 * @since 1.0.0
	 */
	public function install_db(){

		if( ! function_exists( 'maybe_create_table' ) ){
			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
		}

		$sql = "CREATE TABLE {$this->get_table()}(
			id mediumint(9) NOT NULL AUTO_INCREMENT,
			user_id mediumint(9) NOT NULL,
			post_id mediumint(9) NOT NULL,
			date datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
			date_gmt datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
			PRIMARY KEY  ( id, user_id, post_id )
		) {$this->get_charset_collate()};";

		maybe_create_table( $this->get_table(), $sql );
	}

	public function insert( $args ){

		global $wpdb;

		$args = wp_parse_args( $args, array(
			'user_id'			=>	0,
			'post_id'			=>	0,
			'date'				=>	current_time( 'mysql' ),
			'date_gmt'			=>	current_time( 'mysql', 1 )
		) );

		extract( $args );

		return $wpdb->insert(
			$this->get_table(),
			compact( 
				'user_id', 
				'post_id', 
				'date', 
				'date_gmt'
			),
			array( '%d', '%d', '%s', '%s' )
		);
	}

	public function update( $args = array(0) ){

		$args = wp_parse_args( $args, array(
			'user_id'			=>	0,
			'post_id'			=>	0,
			'date'				=>	current_time( 'mysql' ),
			'date_gmt'			=>	current_time( 'mysql', 1 )
		) );

		extract( $args );

		global $wpdb;

		return $wpdb->update(
			$this->get_table(),
			compact( 'date', 'date_gmt' ),
			compact( 'user_id', 'post_id' ),
			array( '%s', '%s' ),
			array( '%d', '%d' )
		);
	}

	public function delete( $user_id, $post_id ){
		global $wpdb;

		return $wpdb->delete(
			$this->get_table(),
			compact( 'user_id', 'post_id' ),
			array( '%d', '%d' )
		);
	}

	public function delete_by_id( $id ){
		global $wpdb;

		return $wpdb->delete(
			$this->get_table(),
			compact( 'id' ),
			array( '%d' )
		);
	}

	public function delete_by_user( $user_id ){
		global $wpdb;

		return $wpdb->delete(
			$this->get_table(),
			compact( 'user_id' ),
			array( '%d' )
		);
	}

	public function delete_by_post_id( $post_id ){
		global $wpdb;

		return $wpdb->delete(
			$this->get_table(),
			compact( 'post_id' ),
			array( '%d' )
		);
	}

	public function get( $args ){

		global $wpdb;

		$limit = '';

		$where = array( 'WHERE 1' );

		$args = wp_parse_args( $args, array(
			'id'				=>	0,
			'user_id'			=>	0,
			'post_id'			=>	0,
			'date'				=>	'',
			'date_gmt'			=>	'',
			'limit'				=>	0,
			'count'				=>	false
		) );

		if( $args['id'] ){
			$where[] = $wpdb->prepare( 'id=%d', $args['id'] );
		}		

		if( $args['user_id'] ){
			$where[] = $wpdb->prepare( 'user_id=%d', $args['user_id'] );
		}

		if( $args['post_id'] ){
			$where[] = $wpdb->prepare( 'post_id=%d', $args['post_id'] );
		}

		if( $args['date'] ){
			$where[] = $wpdb->prepare( 'date=%s', $args['date'] );
		}

		if( $args['date_gmt'] ){
			$where[] = $wpdb->prepare( 'date=%s', $args['date_gmt'] );
		}

		$where = join( ' AND ', $where );
		
		if( (int)$args['limit'] > 0 ){
			$limit = $wpdb->prepare( 'limit %d', $args['limit'] );
		}

		if( ! $args['count'] ){
			return $wpdb->get_results(
				"SELECT * FROM {$this->get_table()} {$where} {$limit}",
				OBJECT
			);
		}
		else{
			return $wpdb->get_var(
				"SELECT COUNT(id) FROM {$this->get_table()} {$where} {$limit}"
			);
		}
	}

	/**
	 *
	 * Get count
	 * 
	 * @param  int $post_id
	 * @return int
	 *
	 * @since 1.0.0
	 * 
	 */
	public function get_count( $post_id ){

		$count = true;

		return $this->get( compact( 'post_id', 'count' ) );
	}

	/**
	 *
	 * Check if given follower_id is following given following_id
	 *
	 * @since  1.0.0
	 * 
	 */
	public function is_liked( $post_id = 0 ){

		$user_id = get_current_user_id();

		if( $post_id == 0 || ! $user_id ){
			return false;
		}

		return $this->get( compact( 'user_id', 'post_id' ) );
	}

	/**
	 *
	 * Get posts
	 * 
	 * @param  array $args
	 * @return $wpdb->get_results()
	 *
	 * @since 1.0.0
	 * 
	 */
	public function get_posts( $args ){
		global $wpdb;

		$args = wp_parse_args( $args, array(
        	'user_id'		=>	get_current_user_id(),
            'post_type'     =>  'video',
            'post_status'   =>  'publish'
		) );

		extract( $args );

        $sql = "SELECT p.ID";

        $sql .= " FROM {$wpdb->prefix}post_like AS pl";
        $sql .= " INNER JOIN";
        $sql .= " {$wpdb->prefix}posts AS p ON pl.post_id = p.ID";
        $sql .= $wpdb->prepare( 
        	" WHERE p.post_status = %s AND p.post_type = %s AND pl.user_id=%d", 
        	$post_status, 
        	$post_type,
        	$user_id
        );
        $sql .= " ORDER BY pl.id DESC";

        return $wpdb->get_results( $sql );
	}
}