<?php

/**
 * Rest API
 *
 * @link       https://1.envato.market/mgXE4y
 * @since      1.0.2
 *
 * @package    WP_Post_Like
 * @subpackage WP_Post_Like/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.2
 * @package    WP_Post_Like
 * @subpackage WP_Post_Like/includes
 * @author     phpface <nttoanbrvt@gmail.com>
 */
if( ! defined( 'ABSPATH' ) ){
	exit;
}

class WP_Post_Like_Rest_API {
    /**
     *
     * Holds the namespace
     * 
     * @var string
     *
     * @since 1.0.2
     */
    private $namespace = 'wp-post-like/';

    /**
     *
     * Holds the verion
     * 
     * @var string
     *
     * @since 1.0.2
     */
    private $version    = 'v1';

    /**
     *
     * Holds the path
     * 
     * @var string
     *
     * @since 1.0.2
     */
    private $path       =   '/like';

    /**
     *
     * Get rest URL
     * 
     * @return string
     *
     * @since 1.0.2
     * 
     */
    public function get_rest_url(){
        return rest_url( "{$this->namespace}{$this->version}{$this->path}" );
    }

    /**
     *
     * Plugin instance
     *
     * @since 1.0.2
     * 
     */
    private function plugin(){
        return WPPL()->get();
    }

    /**
     *
     * Rest API check the queue
     * 
     * @since 1.0.2
     */
    public function rest_api_init(){

        register_rest_route(
            "{$this->namespace}{$this->version}",
            $this->path, 
            array(
                'methods'   =>  WP_REST_Server::CREATABLE,
                'callback'  =>  array( $this , 'create_item' ),
                'args'      =>  array(
                    'post_id' =>  array(
                        'validate_callback' => function( $param, $request, $key ) {

                            if( ! is_numeric( $param ) ){
                                return false;
                            }

                            $post = get_post( $param );

                            if( ! $post || ! in_array( $post->post_type, array( 'video', 'post', 'attachment' ) ) ){
                                return new WP_Error(
                                    'invalid_video',
                                    esc_html__( 'Invalid Video.', 'wp-post-like' )
                                );                    
                            }

                            if( get_post_status( $param ) != 'publish' ){
                                return new WP_Error(
                                    'not_published',
                                    esc_html__( 'This video is not publish.', 'wp-post-like' )
                                );                
                            }

                            return true;
                        }
                    )
                ),
                'permission_callback'   =>  function( $request ){
                    if( ! is_user_logged_in() ){
                        return new WP_Error(
                            'not_logged_in',
                            esc_html__( 'Please log in to like this post.', 'wp-post-like' )
                        );
                    }

                    return true;
                }
            )
        );      
    } 

    /**
     *
     * Create item
     * 
     * @param  WP_Rest_Request $request
     * @return JSON
     *
     * @since 1.0.2
     * 
     */
    public function create_item( $request ){

        $post_id = $request['post_id'];

        $user_id = get_current_user_id();

        if( $this->plugin()->query->is_liked( $post_id ) ){
            $this->plugin()->query->delete( $user_id, $post_id );

            wp_send_json_success( array(
                'code'      =>  'unliked',
                'message'   =>  sprintf(
                    esc_html__( 'You have unliked %s.', 'wp-post-like' ),
                    '<strong>'.get_post( $post_id )->post_title.'</strong>'
                ),
                'count'     =>  number_format_i18n( $this->plugin()->query->get_count( $post_id ) )
            ) );
        }

        $this->plugin()->query->insert( compact( 'user_id', 'post_id' ) );

        wp_send_json_success( array(
            'code'      =>  'liked',
                'message'   =>  sprintf(
                    esc_html__( 'You have liked %s.', 'wp-post-like' ),
                    '<strong>'.get_post( $post_id )->post_title.'</strong>'
                ),
            'count'     =>  number_format_i18n( $this->plugin()->query->get_count( $post_id ) )
        ) );        
    }
}