<?php
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Define the rest functionality.
 *
 *
 * @since      1.0.0
 * @package    WP_Video_Encoder
 * @subpackage WP_Video_Encoder/includes
 * @author     phpface <nttoanbrvt@gmail.com>
 */
class WP_Video_Encoder_Rest{

    /**
     *
     * Holds the namespace
     * 
     * @var string
     */
    private $namespace = 'wp-video-encoder/';

    /**
     *
     * Holds the verion
     * 
     * @var string
     */
    private $version = 'v1';

    /**
     *
     * Plugin instance
     * 
     */
    private function plugin(){
        return wp_video_encoder()->get();
    }

    /**
     *
     * Rest API check the queue
     * 
     * @since 1.0.0
     */
    public function rest_api_init(){

        register_rest_route(
            "{$this->namespace}{$this->version}",
            '/encoding/', 
            array(
                'methods'   =>  WP_REST_Server::READABLE,
                'callback'  =>  function( $data ){
                    return $this->plugin()->queue->run_queue_items();
                },
                'args'      =>  array(
                    'attachment_id' =>  array(
                        'validate_callback' => function( $param, $request, $key ) {
                          return is_numeric( $param );
                        }
                    ),
                    'parent'    =>  array(
                        'validate_callback' => function( $param, $request, $key ) {
                          return is_numeric( $param );
                        }
                    )
                ),
                'permission_callback'   =>  function( $request ){
                    return current_user_can( 'upload_files' );
                }
            )
        );

        register_rest_route(
            "{$this->namespace}{$this->version}",
            '/generate-image/', 
            array(
                'methods'   =>  WP_REST_Server::CREATABLE,
                'callback'  =>  function( $data ){
                    switch ( $data['type'] ) {
                        case 'png':
         
                        break;
                        
                        case 'webp':
                            $results = $this->plugin()->post->generate_attachment_image_webp( $data['attachment_id'] );

                            if( is_wp_error( $results ) ){
                                wp_send_json_error( $results );
                            }

                            if( is_array( $results ) ){

                                $results['image_url'] = wp_get_attachment_image_url( $results['thumbnail_id'] );

                                if( $data['parent'] && get_post_type( $data['parent'] ) == 'video' ){

                                    if( function_exists( 'streamtube_core' ) ){
                                         streamtube_core()->get()->post->update_thumbnail_image_url_2( $data['parent'] , $results['image_url'] );
                                    }   
                                }
                            }

                            wp_send_json_success( $results );
                        break;
                    }
                },
                'args'      =>  array(
                    'attachment_id' =>  array(
                        'validate_callback' => function( $param, $request, $key ) {
                          return is_numeric( $param );
                        }
                    ),
                    'parent'    =>  array(
                        'validate_callback' => function( $param, $request, $key ) {
                            return is_numeric( $param );
                        }
                    ),
                    'type'    =>  array(
                        'validate_callback' => function( $param, $request, $key ) {
                          return is_string( $param ) && in_array( $param, array( 'png', 'webp' ) );
                        }
                    )                    
                ),
                'permission_callback'   =>  function( $request ){
                    return current_user_can( 'edit_others_posts' );
                }
            ),
        );      

        register_rest_route(
            "{$this->namespace}{$this->version}",
            '/queue/', 
            array(
                'methods'   =>  WP_REST_Server::CREATABLE,
                'callback'  =>  function( $data ){
                    return $this->plugin()->queue->requeue_item( $data['attachment_id'] );
                },
                'args'      =>  array(
                    'attachment_id' =>  array(
                        'validate_callback' => function( $param, $request, $key ) {
                          return is_numeric( $param );
                        },
                        'required'  =>  true
                    )
                ),
                'permission_callback'   =>  function( $request ){
                    return current_user_can( 'edit_others_posts' );
                }
            )
        );
    }
}